#ifndef __SD_COUNTER__
#define __SD_COUNTER__

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>
#include <algorithm> // sort

#include "util.h"

namespace SD {

class Counter {
  private:
    std::map<std::string,unsigned int> data_;
  public:
    typedef std::map<std::string,unsigned int>::const_iterator const_iterator;
    Counter(){};
    ~Counter(){};
    Counter( const Counter& obj );
    Counter& operator=( const Counter& obj );
    Counter& operator+=( const Counter& obj );
    void push( const std::string& key, unsigned int val=1 );
    void prune( unsigned int t );
    unsigned int size() const;
    std::vector<std::string> sorted_key() const;
    const_iterator begin() const;
    const_iterator end() const;
};


inline Counter& Counter::operator+=( const Counter& obj ) {

    for ( const_iterator it = obj.begin(); it != obj.end(); ++it ) {
	this->push( it->first ,it->second );
    }
    return *this;
}


inline void Counter::push( const std::string& key, unsigned int val ) {
    
    std::map<std::string,unsigned int>::iterator it = data_.find( key );
    if ( it == data_.end() ) {
	data_.insert( it, std::pair<std::string,unsigned int>( key, val ) );
    }else {
	it->second += val;
    }
}


inline void Counter::prune( unsigned int threshold ) {

    for ( std::map<std::string,unsigned int>::iterator it = data_.begin(); it != data_.end(); ) {
	if ( it->second < threshold ) {
	    data_.erase( it++ );
	}else {
	    it++;
	}
    }
}

    
inline std::vector<std::string> Counter::sorted_key() const {

    std::vector< std::pair<std::string,unsigned int> > v;
    std::copy( data_.begin(), data_.end(), std::back_inserter(v) );
    std::sort( v.begin(), v.end(), cmp_second<std::string,unsigned int>() );
    std::vector<std::string> v2;
    for ( unsigned int i = 0; i < v.size(); ++i ) {
	v2.push_back( v[i].first );
    }
    return v2;
}


inline unsigned int Counter::size() const {

    return data_.size();
}


inline Counter::const_iterator Counter::begin() const {

    return data_.begin();
}


inline Counter::const_iterator Counter::end() const {

    return data_.end();
}


}

#endif
