#include <iostream>
#include <fstream>

#include "util.h"
#include "hmp.h"
#include "double_array.h"
#include "hmp_util.h"

using namespace SD;

int main( int argc, const char** argv ) {

    if ( argc != 3 ) {
	fprintf( stderr, "Usage: hmp_dump <model (binary)> <model (text)>\n" );
	return FAILURE;
    }
    const char* binaryfile = argv[argc-2];
    const char* textfile   = argv[argc-1];
    
    DoubleArray attr_indexer;
    DoubleArray label_indexer;
    unsigned int node_attr_num;
    HMP hmp;
    
    if ( load_model( binaryfile, attr_indexer, node_attr_num, label_indexer, hmp ) == FAILURE ) return FAILURE;

    FILE* os = fopen( textfile, "w" );
    if ( os == NULL ) {
	fprintf( stderr, "cannot open %s\n", textfile );
	return FAILURE;
    }
    
    unsigned int L = label_indexer.key_size();
    for ( unsigned int i = 0; i < node_attr_num; ++i ) {
	for ( unsigned int y = 0; y < L; ++y ) {
	    if ( hmp.node_attr_score( i, y ) != 0.0 ) {
		fprintf( os, "%f\t%s/%s\n",  hmp.node_attr_score(i,y), attr_indexer.key(i), label_indexer.key(y) );
	    }
	}
    }

    for ( unsigned int i = 0; i < attr_indexer.key_size() - node_attr_num; ++i ) {
	for ( unsigned int y1 = 0; y1 < L; ++y1 ) {
	    for ( unsigned int y2 = 0; y2 < L; ++y2 ) {
		if ( hmp.edge_attr_score( i, y1, y2 ) != 0.0 ) {
		    fprintf( os, "%f\t%s/%s/%s\n", hmp.edge_attr_score(i,y1,y2), attr_indexer.key(i+node_attr_num), label_indexer.key(y1), label_indexer.key(y2) );
		}
	    }
	}
    }
    fclose( os );
    
    return SUCCESS;
}
