#ifndef __HMP_UTIL__
#define __HMP_UTIL__

#include <cstring>
#include <set>
#include <string>

#include "util.h"
#include "counter.h"
#include "hmp.h"
#include "double_array.h"

namespace SD {

    
struct Example {
    std::vector<HMP::Input> x;
    std::vector<Label> y;
}; 


inline int get_label_str( const char* line, const char*& label ) {

    for ( const char* s = line + strlen(line) - 1; line <= s; --s ) {
	if ( *s == ' ' || *s == '\t' ) {
	    label = s + 1;
	    return SUCCESS;
	}
    }
    return FAILURE;
}


inline int read_line_preprocess( const char* line, std::set<std::string>& node_attr_set, std::set<std::string>& edge_attr_set, Counter& label_counter ) {

//inline int read_line_preprocess( const char* line, Counter& node_attr_set, Counter& edge_attr_set, Counter& label_counter ) {
    
    const char* label = NULL;
    if ( get_label_str( line, label ) == FAILURE ) {
	return FAILURE;
    }
    if ( strcmp( label, "*" ) != 0 ) {
	label_counter.push( std::string( label ) );
    }

    std::string attr;
    const char* first = line;
    const char* last  = label;
    for ( const char* s = first; s != last; ++s ) {
	if ( *s == ' ' || *s == '\t' ) {
	    if ( attr.length() == 0 ) {
		// white spaces
	    }else if (  attr == "*" ) {
		// value is not defined
	    }else if ( attr[0] == 'U' ) {
		// unigram attribute
		node_attr_set.insert( attr );
		//node_attr_set.push( attr );
	    }else if ( attr[0] == 'B' ) {
		// bigram attribute
		edge_attr_set.insert( attr );
		//edge_attr_set.push( attr );
	    }else {
		return FAILURE;
	    }
	    attr.clear();
	}else {
	    attr.append( 1, *s );
	}
    }
    return SUCCESS;
}


inline int read_line( const char* line, HMP::Input& x, Label& y, const DoubleArray& attr_indexer, unsigned int node_attr_num, const DoubleArray& label_indexer ) {

    const char* label = NULL;
    if ( get_label_str( line, label ) == FAILURE ) {
	return FAILURE;
    }
    y = label_indexer.key_id( label );
    attr_indexer.read( line, label, node_attr_num, x.node_attr, x.edge_attr );

    return SUCCESS;
}


inline int load_model( const char* modelfile, DoubleArray& attr_indexer, unsigned int& node_attr_num, DoubleArray& label_indexer, HMP& hmp ) {

    FILE* is = fopen( modelfile, "rb" );
    if ( is == NULL ) {
	fprintf( stderr, "cannot open %s\n", modelfile );
	return FAILURE;
    }
    if ( attr_indexer.load(is) == FAILURE ) return FAILURE;
    if ( fread( &node_attr_num, sizeof( unsigned int ), 1, is ) != 1 ) return FAILURE;
    if ( label_indexer.load(is) == FAILURE ) return FAILURE;
    if ( hmp.load(is) == FAILURE ) return FAILURE;
    fclose( is );
    return SUCCESS;
}


inline int save_model( const char* modelfile, const DoubleArray& attr_indexer, unsigned int node_attr_num, const DoubleArray& label_indexer, const HMP& hmp ) {

    FILE* os = fopen( modelfile, "wb" );
    if ( os == NULL ) {
	fprintf( stderr, "cannot open %s\n", modelfile );
	return FAILURE;
    }
    if ( attr_indexer.save(os) == FAILURE ) return FAILURE;
    if ( fwrite( &node_attr_num, sizeof( unsigned int ), 1, os ) != 1 ) return FAILURE;
    if ( label_indexer.save(os) == FAILURE ) return FAILURE;
    if ( hmp.save(os) == FAILURE ) return FAILURE;
    fclose( os );
    return SUCCESS;
}

}

#endif
