#ifndef __KBEST__
#define __KBEST__

#include <iostream>
#include <fstream>
#include <vector>

#include "util.h"
#include "chart.h"

namespace SD {

template<class Key, class Value>
class KBest {
  private:
    unsigned int K_;
    std::vector< std::pair<Key,Value> > data_;
  public:
    typedef typename std::vector< std::pair<Key,Value> >::iterator iterator;
    KBest();
    KBest( const KBest& obj );
    ~KBest() {};
    KBest& operator=( const KBest& obj );
    void init( unsigned int K );
    void push( const Key& key, const Value& val );
    iterator begin();
    iterator end();
};


template<class Key, class Value>
KBest<Key,Value>::KBest() {

    data_.reserve( 2048 );
}
    
template<class Key, class Value>
void KBest<Key,Value>::init( unsigned int K ) {

    K_ = K;
    data_.clear();
}


template<class Key, class Value>
void KBest<Key,Value>::push( const Key& key, const Value& val ) {

    if ( data_.size() == K_ && data_.back().second < val ) {
	data_.pop_back();
    }
    
    if ( data_.size() < K_ ) {
	typename std::vector< std::pair<Key,Value> >::iterator lb = lower_bound( data_.begin(), data_.end(), val, cmp_second<Key,Value>() );
	data_.insert( lb, std::pair<Key,Value>( key, val ) );
    }
}


template<class Key, class Value>
typename KBest<Key,Value>::iterator KBest<Key,Value>::begin() {

    return data_.begin();
}


template<class Key, class Value>
typename KBest<Key,Value>::iterator KBest<Key,Value>::end() {

    return data_.end();
}

}

#endif
