#ifndef __RESULT__
#define __RESULT__

#include <iostream>
#include <fstream>
#include <vector>

#include "util.h"
#include "decoder.h"

namespace SD {

class Result {
  private:
    unsigned int example_num_;
    unsigned int token_num_;
    unsigned int correct_token_num_;
    unsigned int iter_num_;
    float edge_rate_;
  public:
    Result();
    Result( const Result& obj );
    ~Result(){};
    Result& operator=( const Result& obj );
    template<class Label, class Model> void update( const Label* y, const Label* y_hat, unsigned int n, const Decoder<Model>& decoder );
    void print( FILE* os=stderr ) const;
};


Result::Result() {
    
    example_num_       = 0;
    token_num_         = 0;
    correct_token_num_ = 0;
    iter_num_          = 0;
    edge_rate_         = 0.0;
}


template<class Label, class Model>
void Result::update( const Label* y, const Label* y_best, unsigned int n, const Decoder<Model>& decoder ) {

    example_num_++;
    token_num_ += n;
    for ( unsigned int i = 0; i < n; ++i ) {
	if ( y[i] == y_best[i] ) ++correct_token_num_;
    }
    iter_num_  += decoder.iter_num();
    edge_rate_ += decoder.edge_rate();
}


void Result::print( FILE* os ) const {

    fprintf( os, "Accuracy: %6.4f (%d/%d)\n", static_cast<float>(correct_token_num_)/static_cast<float>(token_num_), correct_token_num_, token_num_ );
    fprintf( os, "Ave. #iterations : %6.4f\n", static_cast<float>(iter_num_)/static_cast<float>(example_num_) );
}



}

#endif
