#ifndef __SD_SparseMatrix__
#define __SD_SparseMatrix__

#include <iostream>
#include <fstream>
#include <vector>
#include "sparse_vector.h"

namespace SD {
    
class SparseMatrix {
  private:
    unsigned int row_size_;
    std::vector<SparseVector> data_;
  public:
    SparseMatrix();
    SparseMatrix( const SparseMatrix& obj );
    ~SparseMatrix() {};
    SparseMatrix& operator=( const SparseMatrix& obj );
    const SparseVector& operator[]( unsigned int i ) const;
    SparseVector& operator[]( unsigned int i );
    SparseMatrix& operator/=( Real v );
    SparseMatrix& operator-=( const SparseMatrix& obj );
    int load( FILE* is );
    int save( FILE* os ) const;
    void init( unsigned int row_size );
    unsigned int row_size() const;
};


inline SparseMatrix::SparseMatrix() {

    row_size_ = 0;
}


inline SparseMatrix& SparseMatrix::operator=( const SparseMatrix& obj ) {

    if ( this != &obj ) {
	row_size_ = obj.row_size();
	data_.resize( row_size_ );
	for ( unsigned int i = 0; i < row_size_; ++i ) {
	    data_[i] = obj[i];
	}
    }
    return *this;
}


inline const SparseVector& SparseMatrix::operator[]( unsigned int i ) const {

    return data_[i];
}


inline SparseVector& SparseMatrix::operator[]( unsigned int i ) {

    return data_[i];
}


inline SparseMatrix& SparseMatrix::operator/=( Real v ) {

    for ( unsigned int i = 0; i < row_size_; ++i ) {
	data_[i] /= v;
    }
    return *this;
}


inline SparseMatrix& SparseMatrix::operator-=( const SparseMatrix& obj ) {

    if ( this == &obj ) {
	for ( unsigned int i = 0; i < row_size_; ++i ) {
	    data_[i].clear();
	}
    }else {
	for ( unsigned int i = 0; i < row_size_; ++i ) {
	    data_[i] -= obj[i];
	}
    }
    return *this;
}


inline int SparseMatrix::load( FILE* is ) {

    if ( fread( &row_size_, sizeof( unsigned int ), 1, is ) != 1 ) {
	fprintf( stderr, HERE "failed to load\n" );
	return FAILURE;
    }
    data_.resize( row_size_ );
    for ( unsigned int i = 0; i < row_size_; ++i ) {
	if ( data_[i].load( is ) == FAILURE ) return FAILURE;
    }
    return SUCCESS;
}


inline int SparseMatrix::save( FILE* os ) const {

    if ( fwrite( &row_size_, sizeof( unsigned int ), 1, os ) != 1 ) {
	fprintf( stderr, HERE "failed to save\n" );
	return FAILURE;
    }
    for ( unsigned int i = 0; i < row_size_; ++i ) {
	if ( data_[i].save( os ) == FAILURE ) return FAILURE;
    }
    return SUCCESS;
}


inline void SparseMatrix::init( unsigned int row_size ) {
    
    row_size_ = row_size;
    data_.clear();
    data_.resize( row_size_ );
}


inline unsigned int SparseMatrix::row_size() const {

    return row_size_;
}


}

#endif
